/*
 *       Σ
 *  -.
 *
 * kkatarn@rambler.ru, 2010.
 */

#include <math.h>
#include <stdio.h>

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/*
 *   (ޣ ).
 */
static double trapezoid(
	double (*f)(double),	/*   */
	double min,double max,	/*   */
	/*  ,    [a;b] */
	unsigned N)
{
	double x, step = max - min;
	/*   */
	double I = 0.5 * (f(min) + f(max));
	if(N > 1)
	{
		/*     : N-2  */
		step /= N;
		x = min + step;
		for(--N;N--;x += step)
			I += f(x);
	}
	return I * step;
}

/*
 *   (ޣ ).
 */
static double simpson(
	double (*f)(double),	/*   */
	double min,double max,	/*   */
	/* 2N ==  ,    [a;b], N >= 2 */
	unsigned N)
{
	int i = 1;
	double x, step, x_odd, x_even, sum_odd = 0, sum_even = 0;
	if(N < 2)
		return trapezoid(f, min, max, N);
	step = (max - min) / N;
	x_odd = min + step * 0.5;
	x_even = min + step;
	/*     : N   N-1 ޣ  */
	for(--N;N--;x_odd += step, x_even += step) /* N-1  */
	{
		sum_odd += f(x_odd);
		sum_even += f(x_even);
	}
	/*  ޣ  */
	sum_odd += f(x_odd);
	return step * (f(min) + 4 * sum_odd + 2 * sum_even + f(max)) / 6.0;
}

/*
 *     .
 * :
 *	f -  
 *	min, max -  
 *	quad_func -  
 *	upper_delta -      
 *		,    
 */
static double quad(
	double (*f)(double),
	double min,double max,
	double (*quad_func)(double (*f)(double),double,double,unsigned),
	double upper_delta)
{
	int i = 1;
	unsigned N = 2;
	double In, I2n, delta;
	In = quad_func(f, min, max, N);
	do
	{
		N *= 2;
		I2n = quad_func(f, min, max, N);
		delta = fabs(In - I2n);
		/* printf("iter %i: In %g, I2n %g, delta %g\n",i++, In, I2n, delta); */
		In = I2n;
	}	while(delta >= upper_delta);
	return In;
}

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/*
 *       .
 * :
 *	f -  
 *	min, max -  
 *	tolerance -  
 */
double
quad_trapezoid(double (*f)(double),double min,double max,double tolerance)
{
	return quad(f, min, max, trapezoid, tolerance * 3.0);
}

/*
 *       .
 * :
 *	f -  
 *	min, max -  
 *	tolerance -  
 */
double
quad_simpson(double (*f)(double),double min,double max,double tolerance)
{
	return quad(f, min, max, simpson, tolerance * 15.0);
}

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/*
 *  
 */
double f_2_5(double x)
{
	return 2 + sin(2 * sqrt(x));
}

double f_2_12(double x)
{
	/* f(x) * \sqrt{1 + \left(\frac{df}{dx}\right)^2},  f(x) = x^3 */
	double x2 = x * x;
	return x2 * x * sqrt(1 + 9 * x2 * x2);
}

static void do_2_5()
{
	double tol = 0.0001; /*   */
	printf("intergal is %g (tolerance %g)\n",
		quad_simpson(f_2_5, 0, 1, tol), tol);
}

static void do_2_12()
{
	unsigned N = 5;
	printf("area of surface is %g (steps %i)\n",
		2 * M_PI * simpson(f_2_12, 0, 1, N), N);
}

int main(void)
{
	do_2_5();
	do_2_12();
	return 0;
}
